/* /////////////////////////////////////////////////////////////////////////
 * File:        fe.N.c
 *
 * Purpose:     Implementation of the Pantheios fe.N Stock Front-end API.
 *
 * Created:     18th October 2006
 * Updated:     16th January 2007
 *
 * Author:      Matthew Wilson
 *
 * Home:        http://www.pantheios.org/
 *
 * Copyright:   Matthew Wilson, 2006.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the names of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////// */


#include <pantheios/pantheios.h>
#include <pantheios/assert.h>
#include <pantheios/frontend.h>
#include <pantheios/frontends/fe.N.h>

/* /////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

static size_t pantheios_fe_N_countBackEnds_(void)
{
    size_t              n   =   0;
    pan_fe_N_t const    *frontEnd;

    for(frontEnd = &PAN_FE_N_SEVERITY_CEILINGS[0]; 0 != frontEnd->backEndId; ++n, ++frontEnd)
    {}

    return n;
}

static int pantheios_fe_N_calc0Level_(void)
{
    const size_t    numBackEnds     =   pantheios_fe_N_countBackEnds_();
    size_t          n;
    int             severityCeiling =   PAN_FE_N_SEVERITY_CEILINGS[numBackEnds].severityCeiling;

    for(n = 0; n < numBackEnds; ++n)
    {
        pan_fe_N_t const    *frontEnd = &PAN_FE_N_SEVERITY_CEILINGS[n];

        if(frontEnd->severityCeiling > severityCeiling)
        {
            severityCeiling = frontEnd->severityCeiling;
        }
    }

    return severityCeiling;
}

/* /////////////////////////////////////////////////////////////////////////
 * API
 *
 * NOTE: We use *token for storing the (pre-calculated) 0 level
 */

PANTHEIOS_CALL(int) pantheios_fe_init(  void    *reserved
                                    ,   void    **ptoken)
{
    /* We use a union cast here, which we _know_ is safe in this case. */
    union
    {
        void    *token;
        int     level;
    } u;

    PANTHEIOS_MESSAGE_ASSERT("This can only be called when Pantheios is initialising", pantheios_isInitialising());

    STLSOFT_SUPPRESS_UNUSED(reserved);

    u.level = pantheios_fe_N_calc0Level_();

    *ptoken = u.token;

    return 0;
}

PANTHEIOS_CALL(void) pantheios_fe_uninit(void *token)
{
    STLSOFT_SUPPRESS_UNUSED(token);
}

PANTHEIOS_CALL(char const*) pantheios_fe_getProcessIdentity(void *token)
{
    PANTHEIOS_MESSAGE_ASSERT("This can only be called when Pantheios is initialising", pantheios_isInitialising());

    STLSOFT_SUPPRESS_UNUSED(token);

    return PAN_FE_N_PROCESS_IDENTITY;
}

PANTHEIOS_CALL(int) pantheios_fe_isSeverityLogged(  void    *token
                                                ,   int     severity
                                                ,   int     backEndId)
{
    const size_t    numBackEnds     =   pantheios_fe_N_countBackEnds_();
    size_t          n;
    int             severityCeiling =   PAN_FE_N_SEVERITY_CEILINGS[numBackEnds].severityCeiling;

    if(0 == backEndId)
    {
        /* We use a union cast here, which we _know_ is safe in this case. */
        union
        {
            void    *token;
            int     level;
        } u;

        u.token = token;

        severityCeiling = u.level;
    }
    else
    {
        /* Optimise the search, if its index matches the id. */
        if( backEndId >= 1 &&
            (size_t)backEndId <= numBackEnds &&
            PAN_FE_N_SEVERITY_CEILINGS[backEndId - 1].backEndId == backEndId)
        {
            severityCeiling =   PAN_FE_N_SEVERITY_CEILINGS[backEndId - 1].severityCeiling;
        }
        else
        {
            /* Search through the array. If we don't find the backEndId matching, then use
             * the one in .
             */

            for(n = 0; n < numBackEnds; ++n)
            {
                pan_fe_N_t const    *frontEnd = &PAN_FE_N_SEVERITY_CEILINGS[n];

                if(frontEnd->backEndId == backEndId)
                {
                    severityCeiling = frontEnd->severityCeiling;
                    break;
                }
            }
        }
    }

    return (severity & 0x07) <= (severityCeiling & 0x07);
}

/* ////////////////////////////////////////////////////////////////////// */
